#!/bin/python

import socket, string, sys

from FunkConfig import *

class ServerErrorException(Exception): pass

class ClientSong:
    def __init__(self, id, title, artist):
        self.id = id
        self.title = title
        self.artist = artist
    def get_id(self): return self.id
    def get_title(self): return self.title
    def get_artist(self): return self.artist
    def __str__(self):
        return str(self.id) + ": " + self.title + " - " \
               + self.artist

class ClientSongServer:
    def __init__(self, name, enabled):
        self.name = name
        self.enabled = enabled
    def get_name(self): return self.name
    def get_enabled(self): return self.enabled
    def __str__(self):
        return self.name + " " + str(self.enabled)

class FunkClient:
    def __init__(self, config):
        self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.config = FunkConfig(config)
        self.socket.connect((self.config["controlhost"],
                             string.atoi(self.config["controlport"])))
        self.rfile = self.socket.makefile('rb', -1)
        self.wfile = self.socket.makefile('wb', 0)
        self.read_response()

    def send(self, string):
        self.wfile.write(string + "\n")
        self.wfile.flush()

    def readline(self):
        return self.rfile.readline().strip()

    def read_response(self):
        l = self.readline().split(" ",1)
        c = string.atoi(l[0])
        if c != 0: raise ServerErrorException, l[1]

    def read_int(self):
        return string.atoi(self.readline())

    def read_songs(self):
        s = []
        n = self.read_int()
        for x in range(n):
            id = self.read_int()
            title = self.readline()
            artist = self.readline()
            s.append(ClientSong(id, title, artist))
        return s

    def read_songservers(self):
        s = []
        n = self.read_int()
        for x in range(n):
            ss = self.readline().split(" ")
            s.append(ClientSongServer(ss[0], string.atoi(ss[1])))
        return s
    
    def quit(self):
        self.send("quit")
        self.read_response()
        self.wfile.close()
        self.rfile.close()

    def search_title(self, title):
        self.send("search_title " + title)
        self.read_response()
        return self.read_songs()

    def search_artist(self, artist):
        self.send("search_artist " + artist)
        self.read_response()
        return self.read_songs()
    
    def request(self, num):
        self.send("request " + str(num))
        self.read_response()

    def count(self):
        self.send("count")
        self.read_response()
        return self.read_int()

    def list(self, start, end):
        self.send("list " + str(start) + " " + str(end))
        self.read_response()
        return self.read_songs()

    def remove(self, num):
        self.send("remove " + str(num))
        self.read_response()
        
    def queue(self):
        self.send("queue")
        self.read_response()
        return self.read_songs()
        
    def playing(self):
        self.send("playing")
        self.read_response()
        return self.read_songs()

    def skip(self, num):
        self.send("skip " + str(num))
        self.read_response()

    def sslist(self):
        self.send("sslist")
        self.read_response()
        return self.read_songservers()

    def ssenable(self, server):
        self.send("ssenable " + server)
        self.read_response()
    
    def ssdisable(self, server):
        self.send("ssdisable " + server)
        self.read_response()

    def reload(self):
        self.send("reload")
        self.read_response()

    def shutdown(self):
        self.send("shutdown")
        self.read_response()

if __name__ == "__main__":
    # When invoked directly, this module becomes a very simple server
    # management tool. Use it like:
    # python FunkClient.py onyx.config sslist
    # python FunkClient.py onyx.config ssdisable azz
    # python FunkClient.py onyx.config ssenable azz
    # python FunkClient.py onyx.config skip
    # python FunkClient.py onyx.config shutdown
    # python FunkClient.py onyx.config reload
    # python FunkClient.py onyx.config playing

    client = FunkClient(sys.argv[1])
    cmd = sys.argv[2]
    if cmd == "sslist":
        for x in client.sslist():
            r = "disabled"
            if x.get_enabled(): r = "enabled"
            print x.get_name() + ": " + r
    elif cmd == "ssdisable":
        client.ssdisable(sys.argv[3])
        client.reload()
    elif cmd == "ssenable":
        client.ssenable(sys.argv[3])
        client.reload()
    elif cmd == "skip":
        p = client.playing()
        if len(p):
            client.skip(p[0].get_id())
        else:
            client.skip(0)
    elif cmd == "shutdown":
        client.shutdown()
    elif cmd == "reload":
        client.reload()
    elif cmd == "playing":
        p = client.playing()
        if len(p):
            print p[0]
        else:
            print "Nothing playing"

