#!/bin/python
# vim:et:ts=4

import string, os, sys, select
from threading import *
from SocketServer import *
from SongList import *
from RequestQueue import *
from FunkConfig import *
from SongClient import *

class FunkRequestHandler(StreamRequestHandler):
    def respond(self, code, msg):
        line = str(code) + " " + msg
        self.server.log("Sent: " + line)
        self.wfile.write(line + "\n")

    def print_obj(self, value):
        self.wfile.write(str(value) + "\n")

    def print_songs(self, songs):
        self.print_obj(len(songs))
        for x in songs: self.wfile.write(str(x))
        
    def handle(self):
        self.server.log("Got request")
        try:
            self.respond(0, "Hello client!")
            while 1:
                line = self.rfile.readline()
                if line is None: raise IOError, ""
                cmd = line.strip().split(" ", 1)
                self.server.log("Got command: " + str(cmd))
                if cmd[0] == "":
                    self.respond(1, "Null command")
                elif cmd[0] == "help":
                    self.respond(0, "RTFM")
                elif cmd[0] == "quit":
                    self.respond(0, "Byeee.")
                    return
                elif cmd[0] == "search_title":
                    if len(cmd) < 2:
                        self.respond(1, "Bad syntax")
                    else:
                        self.respond(0, "Songs follow")
                        self.server.sl.lock.acquire()
                        res = self.server.sl.search_title(cmd[1])
                        self.server.sl.lock.release()
                        self.print_songs(res)
                elif cmd[0] == "search_artist":
                    if len(cmd) < 2:
                        self.respond(1, "Bad syntax")
                    else:
                        self.respond(0, "Songs follow")
                        self.server.sl.lock.acquire()
                        res = self.server.sl.search_artist(cmd[1])
                        self.server.sl.lock.release()
                        self.print_songs(res)
                elif cmd[0] == "request":
                    if len(cmd) < 2:
                        self.respond(1, "Bad syntax")
                    else:
                        try:
                            n = string.atoi(cmd[1])
                        except ValueError:
                            self.respond(1, "Bad syntax")
                        else:
                            self.server.sl.lock.acquire()
                            song = self.server.sl.get_by_number(n)
                            self.server.sl.lock.release()
                            if song:
                                self.server.rq.lock.acquire()
                                self.server.rq.add_end(song)
                                self.server.rq.lock.release()
                                self.respond(0, "Request added")
                            else:
                                self.respond(1, "Bad song number")
                elif cmd[0] == "count":
                    self.respond(0, "Count follows")
                    self.server.sl.lock.acquire()
                    count = self.server.sl.count()
                    self.server.sl.lock.release()
                    self.print_obj(count)
                elif cmd[0] == "list":
                    cmd = line.strip().split(" ", 2)
                    if len(cmd) < 3:
                        self.respond(1, "Bad syntax")
                    else:
                        try:
                            start = string.atoi(cmd[1])
                            end = string.atoi(cmd[2])
                        except ValueError:
                            self.respond(1, "Bad syntax")
                        else:
                            self.respond(0, "Songs follow")
                            self.server.sl.lock.acquire()
                            res = self.server.sl.get_range(start, end)
                            self.server.sl.lock.release()
                            self.print_songs(res)
                elif cmd[0] == "remove":
                    if len(cmd) < 2:
                        self.respond(1, "Bad syntax")
                    else:
                        try:
                            n = string.atoi(cmd[1])
                        except ValueError:
                            self.respond(1, "Bad syntax")
                        else:
                            self.server.sl.lock.acquire()
                            song = self.server.sl.get_by_number(n)
                            self.server.sl.lock.release()
                            if song:
                                self.server.rq.lock.acquire()
                                self.server.rq.remove(song)
                                self.server.rq.lock.release()
                                self.respond(0, "Removed")
                            else:
                                self.respond(1, "Bad song number")
                elif cmd[0] == "queue":
                    self.respond(0, "Queue follows")
                    self.server.rq.lock.acquire()
                    res = self.server.rq.get_queue()
                    self.server.rq.lock.release()
                    self.print_songs(res)
                elif cmd[0] == "playing":
                    self.respond(0, "Current song follows")
                    self.server.playing_lock.acquire()
                    playing = self.server.playing
                    self.server.playing_lock.release()
                    if playing:
                        self.print_songs([playing])
                    else:
                        self.print_songs([])
                elif cmd[0] == "skip":
                    if len(cmd) < 2:
                        self.respond(1, "Bad syntax")
                    else:
                        try:
                            n = string.atoi(cmd[1])
                        except ValueError:
                            self.respond(1, "Bad syntax")
                        else:
                            self.server.playing_lock.acquire()
                            self.server.skip_lock.acquire()
                            if (not self.server.playing) or (self.server.playing.get_id() == n):
                                self.server.signal_event.set()
                            self.server.skip_lock.release()
                            self.server.playing_lock.release()
                            self.respond(0, "Skipping")
                elif cmd[0] == "reload":
                    self.respond(0, "Reloading playlists")
                    self.server.read_server_lists()
                elif cmd[0] == "shutdown":
                    self.respond(0, "Shutting down server")
                    self.server.shutdown()
                elif cmd[0] == "sslist":
                    self.respond(0, "Songservers follow")
                    self.server.ss_lock.acquire()
                    l = self.server.songservers.keys()
                    self.print_obj(len(l))
                    ss = self.server.songservers
                    for x in ss.keys():
                        self.print_obj(x + " " + str(ss[x].get_enabled()))
                    self.server.ss_lock.release()
                elif cmd[0] == "ssdisable":
                    if len(cmd) < 2:
                        self.respond(1, "Bad syntax")
                    else:
                        self.server.ss_lock.acquire()
                        ss = self.server.songservers
                        if ss.has_key(cmd[1]):
                            ss[cmd[1]].set_enabled(0)
                            self.respond(0, "Server disabled")
                        else:
                            self.respond(1, "No such server")
                        self.server.ss_lock.release()
                elif cmd[0] == "ssenable":
                    if len(cmd) < 2:
                        self.respond(1, "Bad syntax")
                    else:
                        self.server.ss_lock.acquire()
                        ss = self.server.songservers
                        if ss.has_key(cmd[1]):
                            ss[cmd[1]].set_enabled(1)
                            self.respond(0, "Server enabled")
                        else:
                            self.respond(1, "No such server")
                        self.server.ss_lock.release()
                else:
                    self.respond(1, "Unknown command")
        except IOError:
            self.server.log("Broke connection")

class FunkServer(ThreadingMixIn, TCPServer):
    def __init__(self, config):
        self.rq = RequestQueue()
        self.sl = SongList(self.rq)
        self.log_lock = Lock()
        self.playing_lock = Lock()
        self.skip_lock = Lock()
        self.signal_event = Event()
        self.ss_lock = Lock()
        self.playing = None
        self.config = FunkConfig(config)
        TCPServer.__init__(self,
                           (self.config["controlhost"],
                            string.atoi(self.config["controlport"])),
                           FunkRequestHandler)
        self.songservers = {}
        for x in self.config["songservers"].split(" "):
            (name,url) = x.split("=")
            self.songservers[name] = SongClient(url)

    def log(self, *args):
        self.log_lock.acquire()
        print string.join(map(str, args))
        self.log_lock.release()

    def open_shout(self):
        return os.popen(self.config["shoutcommand"], "w")

    def player_process(self):
        shoutin = self.open_shout()
        while 1:
            self.sl.lock.acquire()
            self.playing_lock.acquire()
            self.playing = self.sl.get_next()
            self.playing_lock.release()
            self.sl.lock.release()

            self.signal_event.clear()

            if self.playing is None:
                self.log("No song to play; sleeping until told to skip")
                self.signal_event.wait()
                continue
            
            name = self.playing.get_filename()
            server = self.playing.get_server()
            
            url = self.songservers[server].get_song_url(name)
            self.log("Playing", name, "from", server, "at", url)
            lameout = os.popen("wget -q -O - \"" + url + "\" | lame --mp3input -f -b 128 - -", "r")

            self.log("beginning streaming")
            while 1:
                (infds, outfds, errfds) = select.select([lameout],
                                                        [],
                                                        [lameout, shoutin],
                                                        1)

                if self.signal_event.isSet():
                    self.log("skipped")
                    break
                
                if lameout in infds:
                    try:
                        buf = lameout.read(4096)
                    except IOError:
                        self.log("error while reading source")
                        break
                    if (not buf) or (not len(buf)):
                        self.log("end of file")
                        break
                    try:
                        shoutin.write(buf)
                    except IOError:
                        self.log("error while writing to shoutto; restarting it")
                        shoutin = self.open_shout()
                        break

                if shoutin in errfds:
                    self.log("shoutto died; restarting it")
                    shoutin = self.open_shout()
                    break
                if len(errfds):
                    self.log("some error on one of the fds")
                    break

            self.log("streaming done, closing fds")
            try: lameout.close()
            except IOError: pass
            self.log("fds closed")

    def read_server_lists(self):
        self.log("Reading song lists")
        self.sl.lock.acquire()
        self.ss_lock.acquire()
        req = self.sl.serialise_requests()
        self.sl.clear_playlist()
        for server in self.songservers.keys():
            self.log("Reading song list from " + server)
            for (filename, title, artist) in self.songservers[server].get_songs():
                self.sl.add_song(server, filename, title, artist)
        self.sl.clear_requests()
        self.sl.unserialise_requests(req)
        self.ss_lock.release()
        self.sl.lock.release()

    def save_requests(self):
        self.log("Saving requests")
        self.sl.lock.acquire()
        try:
            f = open(self.config["requests"], "w")
            for x in self.sl.serialise_requests(): f.write(x + "\n")
            f.close()
        except IOError:
            pass
        self.sl.lock.release()

    def load_requests(self):
        self.log("Loading requests")
        self.sl.lock.acquire()
        try:
            self.sl.unserialise_requests(map(string.strip,open(self.config["requests"]).readlines()))
        except IOError:
            pass
        self.sl.lock.release()

    def shutdown(self):
        self.save_requests()
        
    def main(self):
        self.log("Starting up")
        self.read_server_lists()
        self.load_requests()
        self.log("Starting player thread")
        t = Thread(target = self.player_process)
        t.setDaemon(1)
        t.start()
        self.log("Serving")
        self.serve_forever()
        self.log("Terminated")
        
if __name__ == "__main__":
    serv = FunkServer(sys.argv[1])
    serv.main()
